<?php
/******************************************************
 * 框架常用函数文件
 * @version   1.2.0
 *****************************************************/

/* 行错误及异常处理  */
error_reporting(E_ALL);
ini_set('display_errors', 'off');
function graceErrorHandler($code, $message, $file, $line){ 
	if(!PG_SHOWERROR){
        #关闭报错模式
        exit();
	}else{
	    #开启报错模式
        define('systemErrors', json_encode(array($code, $message, $file, $line)));
        include(PG_IN.'templates'.PG_DS.'error.php');
        exit;   
    }
}
//设置用户自定义的错误处理程序，然后触发错误 
set_error_handler("graceErrorHandler");
//注册一个会在PHP中止时执行的函数
register_shutdown_function(function(){ 
    //获取最后发生的错误
	$error = error_get_last(); 
    if (!empty($error)){
		define('systemErrors', json_encode(array('0', $error['message'], $error['file'], $error['line'])));
		include(PG_IN.'templates'.PG_DS.'shutdown.php');
		exit;
    }
});
class graceException extends Exception{
    # 构造函数
    public function __construct($message, $code = null, $previous = null){
        parent::__construct($message, $code, $previous);
    }
	// 调试模式打开后显示页面
	public function showBug(){
		if(PG_DEBUG){ include PG_IN.'templates'.PG_DS.'debug.php'; }
	}
}

//启动404错误页面
function PG_404_Check(){
    //如果控制器内的方法不存在 ,且开启404页面
    if(PG_404){ include(PG_IN.'templates'.PG_DS.'404.php'); exit;  }
}

// 运行追踪
function gracesTrace(){
	if(!PG_TRACE){return false;}
	include PG_IN.'templates'.PG_DS.'trace.php';
}

/* 全局配置文件检查 */
function graceInitConfig(){
	$configFile = PG_IN.'config.php';
	if(is_file($configFile)){return TRUE;}
	file_put_contents($configFile, file_get_contents(PG_IN.'templates'.PG_DS.'config.php'));
}

/* 框架类文件自动加载 */
function __graceAutoLoad($className){
	// 自定义控制器文件加载
	if(substr($className, -10) == 'Controller'){
		$fileUri = PG_PATH.'/'.PG_CONTROLLER.'/'.substr($className, 0, -10).'.php';
		if(is_file($fileUri)){require $fileUri;}
	}
	// 利用命名空间加载其它类文件
	else{
		$fileUri = PG_IN.substr($className, 9).'.php';
		if(PG_DS == '/'){$fileUri = str_replace('\\', '/', $fileUri);}
		if(is_file($fileUri)){require $fileUri;}
	}
}
spl_autoload_register('__graceAutoLoad'); 

/**
 * 功能 : 终止程序运行并输出一段消息
 * @param $msg 文本类型的消息内容 
 */
function pgExit($msg = ''){exit($msg);}

/**
 * 功能 : 打印某个变量
 * @param $var  变量
 * @param $type 默认 false 使用 print_r(), 否则使用 var_dump()
 */
function p($var, $type = false){
	if($type){var_dump($var);}else{print_r($var);}
}

/**
 * 功能 : 获取一个数据表操作对象
 * @param $tableName  数据表名称
 * @param $configName 默认 db , 对应的数据库一级2配置名称
 * @return 数据库操作对象
 */
function db($tableName, $configName = 'db'){
	$conf = sc($configName);
	return phpGrace\tools\db::getInstance($conf, $tableName, $configName);
}

/**
 * 功能 : 获取一个模型
 * @param $modelName  模型名称
 * @param $configName 默认 db , 对应的数据库一级2配置名称
 * @return 模型对象
 */
function model($modelName){
	$modelName = '\\phpGrace\\models\\'.$modelName;
	$model = new $modelName();
	return $model;
}
/**
 * 功能 : 工具实例化函数( 适用于能使用命名空间的工具类 ) 此方法为旧方法，暂且保留
 * @param $args 动态参数
 * @return 对应的工具对象
 */
function t($args) {
	$arguments = func_get_args();
	$className = array_shift($arguments);
	$className = '\\phpGrace\\tools\\'.$className;
	$callback = function() use ($className, $arguments) {
		return new $className(...$arguments);
	};
	return call_user_func_array($callback, $arguments);
}
/**
 * 功能 : 工具实例化函数( 适用于不能使用命名空间的工具类 )
 * @param $args 动态参数
 * @return 对应的工具对象
 */
function tool($args){
	static $staticTools = array();
	$arguments = func_get_args();
	$className = array_shift($arguments);
	$className = '\\'.$className;
	if(empty($staticTools[$className])){
		$fileUri = PG_IN.PG_TOOLS.PG_DS.$className.'.php'; 
		if(!is_file($fileUri)){throw new graceException("类文件 {$className} 不存在");}
		include $fileUri;
		$staticTools[$className] = 1;
	}
	switch(count($arguments)){
		case 0 :
		return new $className();
		break;
		case 1 :
		return new $className($arguments[0]);
		break;
		case 2 :
		return new $className($arguments[0], $arguments[1]);
		break;
		case 3 :
		return new $className($arguments[0], $arguments[1], $arguments[2]);
		break;
		case 4 :
		return new $className($arguments[0], $arguments[1], $arguments[2], $arguments[3]);
		break;
		case 5 :
		return new $className($arguments[0], $arguments[1], $arguments[2], $arguments[3], $arguments[4]);
		break;
	}
}

/**
 * graceRouter 
 * 功能 : 路由解析
 * @return array
*/
function graceRouter (){
    // 解析格式  index-index-11-dd-page10111.htm 类型这样的  控制器-方法名-参数-page页数.网页后缀名
    if(isset($_GET['pathInfo'])){$path = $_GET['pathInfo']; unset($_GET['pathInfo']);}else{$path = 'index'.PG_URL_EXPLODE.'index';}
    if(PG_SUFFIX){$path = str_replace(PG_SUFFIX, '', $path);}
    $router = explode(PG_URL_EXPLODE, $path);
    if(empty($router[0])){array_shift($router);}
    if(PG_ROUTE){
        $routerArray = require(PG_PATH.'/router.php');
        if(array_key_exists($router[0], $routerArray)){
            $newRouter    = array();
            $newRouter[0] = $routerArray[$router[0]][0];
            $newRouter[1] = $routerArray[$router[0]][1];
            if(!empty($routerArray[$router[0]][2]) && is_array($routerArray[$router[0]][2])){
                $newRouter = array_merge($newRouter, $routerArray[$router[0]][2]);
            }
            define("PG_PAGE",  1);
            return $newRouter;
        };
    }
    $router[0] = isset($router[0]) ?  $router[0] : 'index';
    $router[1] = isset($router[1]) ?  $router[1] : 'index';
    for($i = 2; $i < count($router); $i++){
        if(preg_match('/^page(.*)('.PG_SUFFIX.')*$/Ui', $router[$i], $matches)){

            define("PG_PAGE",  intval($matches[1]));
            array_splice($router, $i, 1);
        }
    }
    if(!defined("PG_PAGE")){define("PG_PAGE",  1);}
    return $router;
}


/**
 * 功能 : 修正 POST 参数
 * @param name  post提交的表单里面的name属性名称
 * @param value 修正后的值
 * @return value
 */
function initPOST($name, $value = ''){
	$_POST[$name] = empty($_POST[$name]) ? $value : $_POST[$name];
	return $_POST[$name];
}

/**
 * 当前分组内的自定义配置 [可按照格式进行自定义配置]
 * @param key1 配置名称1
 * @param key2 配置名称2
 * @return 对应配置值
 */
function c($key1, $key2 = null){
	static $config = null;
	if($config == null){$config = require PG_PATH.'/config.php';}
	if(is_null($key1)){return $config;}
	if(is_null($key2)){if(isset($config[$key1])){return $config[$key1];} return null;}
	if(isset($config[$key1][$key2])){return $config[$key1][$key2];}
	return null;
}

/***
 * 读取config.php文件判断文件里面是否有一位数组下标$key，并将$key的值更改为$val
 *
 * @param $key
 * @param $val
 * @return bool
 */
function cSave($key,$val){
    // 1. 读取文件内容
    static $config = null;
    $configFile = PG_PATH.'/config.php';
    if($config == null){$config = require $configFile;}
    if(array_key_exists($key,$config)){
        $config[$key]=$val;
        $content = '<?php return ' . var_export($config, true) . ';';
        file_put_contents($configFile, $content);
        return true;
    }else{
        return  false;
    }
}

/***
 * 修改分组内的config.php文件
 * @param $newArrData 可以为一维数组或者二维数组
 * @return bool
 */
function cUpdate($newArrData){
    // 1. 读取文件内容
    static $config = null;
    $configFile = PG_PATH.'/config.php';
    if($config == null){$config = require $configFile;}
    // 2. 判断新数据的数组下标是否在配置文件的数组下标里面
    if (is_array($newArrData) && !empty($newArrData)) {
        if (count($newArrData) == count($newArrData, COUNT_RECURSIVE)) {
            // 这是一维数组 遍历修改
            foreach ($newArrData as  $k=>$value) {
                // 判断修改的下标是否在配置文件的下标数组里面
                if(array_key_exists($k,$config)){
                    $config[$k]=$value;
                }
            }
        } else {
            // 这是二维数组 遍历修改 有修改项则修改，不存在的则不动
            foreach ($newArrData as $k1 => $v1) {
                if (array_key_exists($k1, $config)) {
                    foreach ($v1 as $key2 => $v2) {
                        if (array_key_exists($key2, $config[$k1])) {
                            $config[$k1][$key2] = $v2;
                        }
                    }
                }
            }
            // 3.将更新后的数据写回到文件中
            $content = '<?php return ' . var_export($config, true) . ';';
            file_put_contents($configFile, $content);
            // 修改成功后给个返回真
            return true;
        }
    }else{
        // 不是数组，无法修改，返回假
        return false;
    }
}

/**
 * 全局配置 [可按照格式进行自定义配置]
 * @param $key 配置名称1
 * @param $key 配置名称2
 */
function sc($key1 = null, $key2 = null){
	static $config = null;
	if($config == null){
		$config = require PG_IN.'config.php';
	}
	if(is_null($key1)){return $config;}
	if(is_null($key2)){if(isset($config[$key1])){return $config[$key1];} return null;}
	if(isset($config[$key1][$key2])){return $config[$key1][$key2];}
	return null;
}


/**
 * 模块配置
 * 读取 /phpGrace/configs/配置名称[ 通常与模块名称同名 ].php 后添加自己的配置数据；
 * @param $configName  模块名称 无.php
 * @param $ke1  配置名称1
 * @param $key2 配置名称2
 * @return void
 */
function config($configName,$key1=null,$key2=null){
    static $config = null;
    if($config == null){
        $config = require PG_IN.'configs/'.$configName.'.php';
    }
    if(is_null($key1)){return $config;}
    if(is_null($key2)){if(isset($config[$key1])){return $config[$key1];} return null;}
    if(isset($config[$key1][$key2])){return $config[$key1][$key2];}
    return null;
}

/**
 * 时间、内存开销计算
 * @return array(耗时[毫秒], 消耗内存[K])
 */
function pgCost(){
	return array(
		round((microtime(true) - PG_START_TIME) * 1000, 2),
		round((memory_get_usage() - PG_START_MEMORY) / 1024, 2)
	);
}

/**
 * 开启 session
 */
function startSession(){
	switch(PG_SESSION_TYPE){
		case 'file' :
			if(!is_dir(PG_SESSION_DIR)){mkdir(PG_SESSION_DIR, 0777, true);}
			session_save_path(PG_SESSION_DIR);
		break;
		case 'memcache' :
			ini_set("session.save_handler", "memcache");
			ini_set("session.save_path", PG_SESSION_HOST);
		break;
		case 'redis':
			ini_set("session.save_handler", "redis");
			ini_set("session.save_path", PG_SESSION_HOST);
		break;
		default:
			if(!is_dir(PG_SESSION_DIR)){mkdir(PG_SESSION_DIR, 0777, true);}
			session_save_path(PG_SESSION_DIR);
	}
	session_start();
	session_write_close();
}

/**
 * 设置 session
 * @param $name session 名称
 * @param $val  对应的值
 */
function setSession($name, $val){
	session_start();
	if(is_array($val)){
		foreach($val as $k => $v){$_SESSION[$k] = $v;}
	}else{
		$_SESSION[$name] = $val;
	}
	session_write_close();
}

/**
 * 获取 session
 * @param $name session 名称
 */
function getSession($name){
	if(isset($_SESSION[$name])){return $_SESSION[$name];} 
	return null;
}

/**
 * 销毁指定的 session
 * @param $name session 名称
 */
function removeSession($name){
	session_start();
	if(is_array($name)){
		foreach($name as $k){
			if(isset($_SESSION[$k])){unset($_SESSION[$k]);}
		}
	}else{
		if(isset($_SESSION[$name])){unset($_SESSION[$name]);}
	}
	session_write_close();
}

/**
 * 设置 cookie
 * @param $name   cookie 名称
 * @param $val    对应的值
 * @param $expire 有效时间 单位秒 默认365天的秒数
 */
function pgSetCookie($name, $val, $expire = 31536000){
	$expire += time();
	@setcookie($name, $val, $expire, '/');
	$_COOKIE[$name] = $val;
}

/**
 * 获取 cookie
 * @param $name cookie 名称
 * @return 具体 cookie 值或 null
 */
function pgGetCookie($name){if(isset($_COOKIE[$name])){return $_COOKIE[$name];} return null;}

/**
 * 删除指定 cookie
 * @param $name cookie 名称
 */
function pgRemoveCookie($name){
	setcookie($name, 'null', time() - 1000, '/');
}

/**
 * 获取语言
 * @param $key 语言包键名称
 * @return 具体的值或者null 
 */
function lang($key){
	static $Lang = null;
	if(is_null($Lang)){
		$langName = empty($_COOKIE['phpGraceLang']) ? 'zh' : $_COOKIE['phpGraceLang'];
		$langFile = PG_PATH.'/'.PG_LANG_PACKAGE.'/'.$langName.'.php';
		if(is_file($langFile)){
			$Lang = require $langFile;
		}else{
			throw new graceException('语言包文件不存在',100002);
		}
	}
	if(isset($Lang[$key])){return $Lang[$key];}
	return null;
}
/**
 * 路径解析
 * @param $c      控制器名称
 * @param $m      方法名称
 * @param $params 参数 : 数组或字符串模式
 * @param $page   页码
 * @return 具体的值或者null 
 */
function u($c, $m, $params = '', $page = null){
	$suffix = PG_SUFFIX ? PG_SUFFIX : '';
	$page = $page != null ? PG_URL_EXPLODE.'page'.$page : '';
	if(is_array($params)){
		return PG_SROOT.$c.PG_URL_EXPLODE.$m.PG_URL_EXPLODE.implode(PG_URL_EXPLODE, $params).$page.$suffix;
	}else{
		if($params != ''){
			return PG_SROOT.$c.PG_URL_EXPLODE.$m.PG_URL_EXPLODE.$params.$page.$suffix;
		}else{
			return PG_SROOT.$c.PG_URL_EXPLODE.$m.$page.$suffix;
		}
	}
}

/**
 * 路由解析，u的别名
 */
function url($c, $m, $params = '', $page = null){
    return u($c, $m, $params, $page);
}
/**
 * 生成一个 token [ cookie 模式 ]
 * @return 具体的值或者null
 */
function setToken(){
	$token = uniqid();
	pgSetCookie('__gracetoken__', $token);
	return $token;
}

/**
 * 获取 token [ cookie 模式 ]，并销毁
 * @return 具体的值或者null
 */
function getToken(){
	$token = pgGetCookie('__gracetoken__');
	pgRemoveCookie('__gracetoken__');
	return $token;
}

/**
 * 去除空白字符
 * @param $str 需要替换的字符串
 * @return     替换后的结果
 */
function trimAll($str){
    $qian=array(" ","　","\t","\n","\r");
    $hou=array("","","","","");
    return str_replace($qian,$hou,$str); 
}

/**
 * 输出 option 选中状态
 * @param $val1 比对值1
 * @param $val2 比对值2
 */
function isSelected($val1, $val2){if($val1 == $val2){echo ' selected="selected"';}}

/**
 * 输出 radio ,checkbox 选中状态
 * @param $val1 比对值1
 * @param $val2 比对值2
 */
function isChecked($val1,$val2){ if($val1 == $val2){ echo "checked"; } }
function ischeckInArr($val1,$arr){ if (in_array($val1,$arr)){echo 'checked';} }
/**
 * 将数值转换为 select 菜单的 option
 * @param $data      数据数组
 * @param $currentId 默认选项对应的键
 */
function dataToOption($data, $currentId = 0){
	foreach($data as $k => $v){
		if($currentId == $k){
			echo "<option value=\"{$k}\" selected=\"selected\">{$v}</option>".PHP_EOL;
		}else{
			echo "<option value=\"{$k}\">{$v}</option>".PHP_EOL;
		}
	}
}

/** 
 * 规划缓存命名 
 * @param $name      缓存名称
 * @param $parameter 缓存影响参数
 * @param $isSuper   是否为全局缓存
 * @return 缓存名称
 */
function graceCacheName($name, $parameter = '', $isSuper = true){
	$cacheConfig = sc('cache');
	$parameter   = is_array($parameter) ? implode('_', $parameter) : $parameter;
	$cacheName   = $isSuper ? $cacheConfig['pre'].$name.$parameter : $cacheConfig['pre'].PG_C.'_'.PG_M.'_'.$name.$parameter;
	if(empty($cacheConfig['name2md5'])){ return $cacheName; }
	return md5($cacheName);
}
/**
 * 判断是否为移动端访问
 * @return bool 
 * */
function isMobile() {
    if (isset($_SERVER['HTTP_VIA']) && stristr($_SERVER['HTTP_VIA'], "wap")) { 
        return true; 
    } elseif (isset($_SERVER['HTTP_ACCEPT']) && strpos(strtoupper($_SERVER['HTTP_ACCEPT']), "VND.WAP.WML")) {
        return true;
    } elseif (isset($_SERVER['HTTP_X_WAP_PROFILE']) || isset($_SERVER['HTTP_PROFILE'])) {
        return true;
    } elseif ( isset($_SERVER['HTTP_USER_AGENT']) && preg_match('/(blackberry|configuration\/cldc|hp |hp-|htc |htc_|htc-|iemobile|kindle|midp|mmp|motorola|mobile|nokia|opera mini|opera |Googlebot-Mobile|YahooSeeker\/M1A1-R2D2|android|iphone|ipod|mobi|palm|palmos|pocket|portalmmm|ppc;|smartphone|sonyericsson|sqh|spv|symbian|treo|up.browser|up.link|vodafone|windows ce|xda |xda_)/i', $_SERVER['HTTP_USER_AGENT'])) {
        return true;
    } else {
        return false; 
    }
}
/**
 * 判断当前协议是否为HTTPS
 * @return bool
 */
function isHttps() {
	if (!empty($_SERVER['HTTPS']) && strtolower($_SERVER['HTTPS']) !== 'off') {
		return true;
	} elseif (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] === 'https') {
		return true;
	} elseif (!empty($_SERVER['HTTP_FRONT_END_HTTPS']) && strtolower($_SERVER['HTTP_FRONT_END_HTTPS']) !== 'off') {
		return true;
	}
	return false;
}
/**
* 检测是否AJax请求
* @return bool
*/
function isAjax() {
    if (isset($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') {return true;}
    return false;
}
/**
* 检测是否Post请求
* @return bool
*/
function isPost() {
    if( $_SERVER['REQUEST_METHOD'] == 'POST' && ( empty($_SERVER['HTTP_REFERER']) || preg_replace("~https?:\/\/([^\:\/]+).*~i", "\\1", $_SERVER['HTTP_REFERER']) == preg_replace("~([^\:]+).*~", "\\1", $_SERVER['HTTP_HOST'])) ) {return true;}
    return false;
}
/**
* 检测是否Get请求
* @return bool
*/
function isGet() {
    if($_SERVER['REQUEST_METHOD'] == 'GET') {return true;}
    return false;
}
/**
 * 检测是否Patch请求
 * @return bool
 */
function isPatch() {
    if($_SERVER['REQUEST_METHOD'] == 'PATCH') {return true;}
    return false;
}
/**
 * 检测是否OPTIONS请求
 * @return bool
 */
function isOptions() {
    if($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {return true;}
    return false;
}
## -----------------------------------------下面是自己额外添加的------------------------------------------------

/***
 * 递归函数，实现无限极分类列表
 * @param $list 数组
 * @param $pid 父级ID
 * @param $level 级别
 * @return array
 */
function get_cate_list($list,$pid = 0,$level =0){
    static $tree  = array();
    foreach ($list as $row) {
        if($row['pid']==$pid){
            $row['level'] = $level;
            $tree[] = $row;
            get_cate_list($list,$row['id'],$level+1);
        }
    }
    return $tree;
}


/***
 * 引用方式实现 父子级树状结构  效率比递归高
 * @param $list
 * @return array|mixed
 */
function get_tree_list($list){
    //将每条数据中的id的值作为其下标
    $temp = [];
    foreach ($list as $v) {
        $v['son'] = [];
        $temp[$v['id']] = $v;
    }
    //获取分类树
    foreach ($temp as $k => $v ) {
       $temp[$v['pid']]['son'][] = &$temp[$v['id']];
    }
    return isset($temp[0]['son']) ? $temp[0]['son']:[];
}

/***
 * 删除文件夹下所有文件
 * @param $folder 文件夹名
 * @return void
 */
function delSessionFloder($folderName)
{
    // 获取文件夹内所有文件
    $files = glob($folderName.'/*');
    // 循环删除每个文件
    foreach($files as $file){
        if(is_file($file)){
            unlink($file);
        }
    }
}


/**
 * 判断浏览器类型
 * @return string
 */
function edgeType(){
    $user_agent = $_SERVER['HTTP_USER_AGENT'];
    $type = null;
    if (strpos($user_agent, 'MSIE') !== false) {
        $type  ='Internet Explorer浏览器';
    } elseif (strpos($user_agent, 'Firefox') !== false) {
        $type  ='Firefox浏览器';
    } elseif (strpos($user_agent, 'Chrome') !== false) {
        $type  = 'Chrome浏览器';
    } elseif (strpos($user_agent, 'Safari') !== false) {
        $type  = 'Safari浏览器';
    } elseif (strpos($user_agent, 'Opera') !== false) {
        $type  = 'Opera浏览器';
    } else {
        $type  = '无法识别';
    }
    return $type;
}

/***
 * 输出分页信息
 * 调用参考：
 *  <div class="layui-card-body ">
        <?php pageList($this->roleinfos[1]);?>
 *  </div>
 * @param $pageArrayObj  $this->roleinfos[1]
 * @return void
 */
function pageList($pageArrayObj){
    $html = '<div class="page grace-pager">';
    $html.='<a href="'.$pageArrayObj->firstPage.'">首页</a>';
    $html.='<a href="'.$pageArrayObj->prePage.'">上一页</a>';
    foreach($pageArrayObj->listPage as $k => $v){
        if($k == $pageArrayObj->currentPage){
            $html.= '<a href="'.$v.'" class="grace-current">'.$k.'</a>';
        }else{
            $html.= '<a href="'.$v.'">'.$k.'</a>';
        }
    }
    $html.=' <a href="'.$pageArrayObj->nextPage.'">下一页</a>';
    $html.='<a href="'.$pageArrayObj->lastPage.'">尾页</a>';
    $html.='</div>';
    echo $html;
}

/**
 * 获取文件大小
 * @param $filesize
 * @return string
 */
function getSize($filesize) {
    if($filesize>=1099511627776){
        $filesize = round($filesize / 1099511627776 * 100) / 100 . 'TB';
    }elseif($filesize >= 1073741824) {
        $filesize = round($filesize / 1073741824 * 100) / 100 . 'GB';
    } elseif($filesize >= 1048576) {
        $filesize = round($filesize / 1048576 * 100) / 100 . 'MB';
    } elseif($filesize >= 1024) {
        $filesize = round($filesize / 1024 * 100) / 100 . 'KB';
    } else {
        $filesize = $filesize . 'B';
    }
    return $filesize;
}

/***
 * 秒转换为 小时：分钟：秒数的格式
 * @param $seconds 秒数
 * @return string 小时分钟秒数格式
 */
function secondsToTime($seconds) {
    $hours = floor($seconds / 3600);
    $minutes = floor(($seconds % 3600) / 60);
    $seconds = $seconds % 60;
    return sprintf("%02d:%02d:%02d", $hours, $minutes, $seconds);
}

/***
 * 解POST实体转义
 * @param $data
 * @return array|string|string[]
 */
function jiepost($data){
   return str_replace(array('&lt;','&gt;', '&quot;', ''),array('<','>', '"', "'"), $data);
}

/**
 * POST实体转义
 */
function postFilter(){
    $_POST = str_replace(array('<','>', '"', "'"),array('&lt;','&gt;', '&quot;', ''),  $_POST);
}

/***
 * 数组判断且初始化.如果存在则返回该值，不存在则返回默认值
 * @param $arr 数组
 * @param $key 数组的下标
 * @param $default 如果该数组下标的值不存在，则设置的默认值
 * @return mixed|string
 */
function array_value($arr, $key, $default = '') {
    return isset($arr[$key]) ? $arr[$key] : $default;
}
/***
 * 发送邮箱 使用的是composer 安装的Swiftmailer
 * 参考：https://www.helloweba.net/php/457.html
 * @param $mailConfig 邮箱配置数组字段
 * @return int 成功则返回数字1
 */
function sendMailer($mailConfig){
    // 基础配置赋值
    $swiftMail_host     =  array_value($mailConfig,'host'       ,c('email','smtp_host')     ) ; #主机
    $swiftMail_port     =  array_value($mailConfig,'port'       ,c('email','smtp_port')     ) ; #端口
    $swiftMail_username =  array_value($mailConfig,'send_email' ,c('email','send_email')    );#邮箱用户名
    $swiftMail_password =  array_value($mailConfig,'send_miyue' ,c('email','send_miyue')    );#邮箱密码或者授权码
    $swiftMail_title    =  array_value($mailConfig,'title');   #邮箱发送的标题
    $swiftMail_uname    =  array_value($mailConfig,'uname'      ,c('website','webname')     ); #邮箱自定义的昵称
    $swiftMail_tomail   =  array_value($mailConfig,'tomail') ;  #邮件收件箱，可以为数组
    $swiftMail_body     =  array_value($mailConfig,'body');    # 邮箱发送的内容
    // 基本配置
    $transport = (new Swift_SmtpTransport($swiftMail_host, $swiftMail_port,'ssl')) //smtp服务器和端口和加密--加密必须有不然布置到服务器会异常
        ->setUsername($swiftMail_username) // 配置用户名
        ->setPassword($swiftMail_password) // 配置密码或者授权码
    ;
    // 创建mailer对象
    $mailer = new Swift_Mailer($transport);
    //创建消息对象
    $message = (new Swift_Message($swiftMail_title)) // 发件标题
        ->setFrom([$swiftMail_username => $swiftMail_uname])//自定义发件人的名称
        ->setTo($swiftMail_tomail) //收件人邮箱
        ->setBody($swiftMail_body,'text/html') // 发件内容
    ;
    // 发送邮件 成功则返回数字：1 ,异常返回false
    try {
       return $mailer->send($message);
    }catch(Exception  $e){
       return  false;
    }
}

/**
 * 邮箱模板{关键字} 替换为其他值，返回被替换后的值
 * @param $TemplateText
 * @param $ReplaceArr
 * @return array|mixed|string|string[]
 */
function templateReplace(&$TemplateText,$ReplaceArr){
    // 准备替换邮箱模板的值
    foreach ($ReplaceArr as $k  => $v ) {
        $TemplateText = str_replace("{".$k."}", $v, $TemplateText);
    }
    return $TemplateText;
}
/***
 * 跳转到网页
 * @param $url
 * @return void
 */
function jump($url){
    header('location:'.$url);
}

/***
 * 检查参数是否存在，如果存在则赋值 不存在则返回JSON错误消息提示
 * @param $parma
 * @param $tipinfo
 * @return void
 */
function checkParmaInti($parma,$tipinfo)
{
    return isset($parma)?$parma: pgExit(json_encode(array('status' => 'error', 'data' => $tipinfo),JSON_UNESCAPED_UNICODE));
}

/**
 * 压缩图片 
 * @param string $source 图片源文件路径
 * @param string $destination 压缩后图片的目标保存路径
 * @param int $quality 压缩质量，取值范围0-100，质量越低文件越小
 * @return string 返回压缩后图片的保存路径
 */
function compressImage($source, $destination, $quality) {
    // 获取源图片的格式和尺寸信息
    $info = getimagesize($source); 
    // 根据源图片的格式进行图片资源的创建
    if ($info['mime'] == 'image/jpeg') {
        $image = imagecreatefromjpeg($source);
    } elseif ($info['mime'] == 'image/gif') {
        $image = imagecreatefromgif($source);
    } elseif ($info['mime'] == 'image/png') {
        $image = imagecreatefrompng($source);
    } 
    // 将图片资源以指定的质量保存为JPEG格式到目标路径
    imagejpeg($image, $destination, $quality); 
    return $destination;
}
/**
 * 将多维数组扁平化为一维数组。
 * @param array $array 需要扁平化的数组。
 * @return array 扁平化后的一维数组。
 */
function array_flatten($array) {
    $result = []; // 初始化扁平化后的数组
    foreach ($array as $value) { // 遍历输入数组的每个值
        if (is_array($value)) { // 如果当前值是数组，则递归扁平化这个子数组
            $result = array_merge($result, array_flatten($value));
        } else {
            $result[] = $value; // 如果当前值不是数组，则将其添加到结果数组中
        }
    }
    return $result; // 返回扁平化后的一维数组
}

